#!/bin/bash

logfile=./exp.log
exec 2>${logfile}

dbsize() {
    ls -l db$1 | awk '{printf("%d", $5)}'
}

matrix() { # 実験パターンテーブル表示用
cat << --
        INDEX   DELETE  TABLE   INDEX   |
 db1    n       Y       n       n       |
 db2    n       n       Y       n       |
 db3    Y       Y       n       n       |
 db4    Y       n       Y       n       |
 db5    Y       n       n       Y       |
 db6    Y       Y       n       Y       |
 db7    Y       Y       Y       n       |
--
}

result() {
    local i
    printf "    init    insr    drop    vacu\n"
    for i in {0..6}
    do
        printf " %7d %7d %7d %7d\n" ${init[$i]} ${insr[$i]} ${drop[$i]} ${vacu[$i]}
    done
}

createtable() {
    echo "Create Table db$1" >&2
    time sqlite3 db$1 "CREATE TABLE tbl1 ( col1 TEXT );"
    ls -l db${1}
    echo
}

createindex() {
    echo "Create Index db$1" >&2
    time sqlite3 db$1 "CREATE INDEX tbl1idx1 ON tbl1 (col1);"
    ls -l db$1
    echo
}

insert() {
    local count=4
    local str=()
    local i sz0 sz

    echo "Insert into db$1, until $count times expansion" >&2

    sz0=$(dbsize $1)
    printf "\tdb size=%d " $sz0
    i=0
    while [ $i -lt $count ]
    do # sqlite3へは、ヒアドキュメントでSQL文を入力
        str=( $( cat /dev/urandom | LANG=C tr -dc '!-&(-[]-~' | fold -w 64 | head -n 16 ))
        # 以下の手順で、64文字のランダムアスキー文字列を生成
            # /dev/urandomは、ランダムなビットパターンを生成
            # tr -dc ... で、文字列に含めたくない文字を排除
            # fold -w で、1行の長さを指定
        #cat << --  # この行は実際にinsert処理をしない時に、確認するためのデバッグ用
        time sqlite3 db$1 << --
        INSERT INTO tbl1 VALUES('${str[0]}');  INSERT INTO tbl1 VALUES('${str[1]}');
        INSERT INTO tbl1 VALUES('${str[2]}');  INSERT INTO tbl1 VALUES('${str[3]}');
        INSERT INTO tbl1 VALUES('${str[4]}');  INSERT INTO tbl1 VALUES('${str[5]}');
        INSERT INTO tbl1 VALUES('${str[6]}');  INSERT INTO tbl1 VALUES('${str[7]}');
        INSERT INTO tbl1 VALUES('${str[8]}');  INSERT INTO tbl1 VALUES('${str[9]}');
        INSERT INTO tbl1 VALUES('${str[10]}');  INSERT INTO tbl1 VALUES('${str[11]}');
        INSERT INTO tbl1 VALUES('${str[12]}');  INSERT INTO tbl1 VALUES('${str[13]}');
        INSERT INTO tbl1 VALUES('${str[14]}');  INSERT INTO tbl1 VALUES('${str[15]}');
--
        sz=$(dbsize $1)
        if [ $sz -gt $sz0 ]; then
            let ++i
            printf " %d" $sz
            sz0=$sz
        else
            echo -n " ."
        fi
        sleep 1
    done
    echo
    echo "Inserting completed" >&2
    ls -l db$1
    echo
}

delete() {
    echo "Delte db$1" >&2
    time sqlite3 db$1 "DELETE FROM tbl1;"
    ls -l db$1
    echo
}

droptable() {
    echo "Drop Table db$1" >&2
    time sqlite3 db$1 "DROP TABLE tbl1;"
    ls -l db$1
    echo
}

dropindex() {
    echo "Drop Index db$1" >&2
    time sqlite3 db$1 "DROP INDEX tbl1idx1;"
    ls -l db$1
    echo
}

vacuum() {
    echo "Vacuum db$1" >&2
    echo -n "vacuuming "
    ls -il db$1
    time sqlite3 db$1 "VACUUM;"
    echo -n "vacuumed  "
    ls -il db$1
    echo
}

##
declare -a init=()
declare -a insr=()
declare -a drop=()
declare -a vacu=()

echo "Clear DBs"
rm db?

# まずは、db1にテーブルスキームだけ作り、db3に複製を作る
# sqliteは、静止状態なら、dbファイルをコピーして複製できる;-)
echo "Create DB and Table"
createtable 1
cp db1 db3

echo "Create Index" # db3は、index付きdbの祖先
createindex 3

init=( $(dbsize 1) $(dbsize 1) $(dbsize 3) $(dbsize 3) $(dbsize 3) $(dbsize 3) $(dbsize 3) )

echo "Insert w/o index" # db1にデータを挿入。挿入後db2に複製
insert 1
cp db1 db2

echo "Insert w/ index" # db3にデータを挿入。挿入後db4,5,6,7に複製
insert 3
cp db3 db4
cp db3 db5
cp db3 db6
cp db3 db7

for db in {1..7}; do insr+=($(dbsize $db)); done # insert後のサイズを記録
echo ${init[@]}

echo "Delete" # db1,3,6,7は、delete
delete 1
delete 3
delete 6
delete 7

echo "Drop Index" # db5,6は、drop index
dropindex 5
dropindex 6

echo "Drop Table" # db2,4,7は、drop table
droptable 2
droptable 4
droptable 7


for db in {1..7}; do drop+=($(dbsize $db)); done # 削除処理後のサイズを記録

echo "Vacuum"
vacuum 1
vacuum 2
vacuum 3
vacuum 4
vacuum 5
vacuum 6
vacuum 7

for db in {1..7}; do vacu+=($(dbsize $db)); done # vacuum処理後のサイズを記録

# 結果を出力
echo "All done"
ls -l db?

echo
paste <( matrix ) <( result )

